﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;


public class WandScript : ItemScript {

	public GameObject wandPrefab;
	public Wand wand;
	public Transform magicOrigin;

	PlayerBase playerBase;

	// Use this for initialization
	void Start () {
		base.Start ();
	}

	// Update is called once per frame
	void Update () {
		base.Update ();

		if (isActive)
			UpdateActiveWand ();
	}

	void OnTriggerStay (Collider col) {
		if (col.gameObject.tag == "Player"){
			if (Input.GetKeyDown (KeyCode.R)) {
				InitializeWand (col.gameObject);
				ItemDisplays.itemDisplay.AssignItem (gameObject); // need to check what this does
			}
		}
	}

	void InitializeWand (GameObject player) {
		myOwner = player;
		playerBase = myOwner.GetComponent<PlayerBase> ();

		// set bools
		pickedUp = true;
		isActive = true;
		gameObject.GetComponent<SphereCollider> ().enabled = false;

		// set transform variables
		transform.parent = playerBase.rightHand;
		transform.localPosition = Vector3.zero;
		transform.localScale = Vector3.one;
		transform.localEulerAngles = new Vector3 (-90f, 0f, 0f);

		// set external script variables
		playerBase.activeWandObj = gameObject;
		playerBase.spawnPoint = magicOrigin;

		// handle player base list
		if (playerBase.activeItem != null)
			playerBase.DropItem (playerBase.activeItem);
		playerBase.inventory [playerBase.activeItemNum] = gameObject;
		playerBase.activeItem = gameObject;
	}

	void UpdateActiveWand () {
		playerBase.activeWandObj = gameObject;
		playerBase.spawnPoint = magicOrigin;
	}


	[ExecuteInEditMode]
	void OnValidate () {

		if (wand.rarity == Rarity.Basic) {
			wand.iconColor = new Color (0.541f, 0.565f, 0.580f, 1.000f);
			wand.baseDamage = 1f;
			wand.maxDamage = 20f;
		} else if (wand.rarity == Rarity.Normal) {
			wand.iconColor = new Color (0.314f, 0.620f, 0.008f, 1.000f);
			wand.baseDamage = 2f;
			wand.maxDamage = 40f;
		} else if (wand.rarity == Rarity.Unique) {
			wand.iconColor = new Color (0.000f, 0.576f, 0.725f, 1.000f);
			wand.baseDamage = 3f;
			wand.maxDamage = 60f;
		} else if (wand.rarity == Rarity.Super) {
			wand.iconColor = new Color (0.675f, 0.306f, 0.831f, 1.000f);
			wand.baseDamage = 4f;
			wand.maxDamage = 80f;
		} else {
			wand.iconColor = new Color (0.855f, 0.518f, 0.239f, 1.000f);
			wand.baseDamage = 5f;
			wand.maxDamage = 100f;
		}

		if (wand.damage < wand.baseDamage)
			wand.damage = wand.baseDamage;
		else if (wand.damage > wand.maxDamage)
			wand.damage = wand.maxDamage;
	}
}


/// <summary>
/// The primary type of a wand: Scatter, Burst, Beam, Orb.
/// </summary>
[System.Serializable]
public enum WandType {Scatter, Burst, Beam, Orb}
/// <summary>
/// The sub type of a wand: None, Auto, Charged.
/// </summary>
[System.Serializable]
public enum WandSubType {None, Auto, Charged}
/// <summary>
/// Defines the rarity of an item: Basic (Gray), Normal (Green), Unique (Blue), Super (Purple), Extraordinary (Gold).
/// </summary>
[System.Serializable]
public enum Rarity {Basic, Normal, Unique, Super, Extraordinary}

/// <summary>
/// Base class for wands.
/// </summary>
[System.Serializable]
public class Wand {
	[ReadOnly]
	public Color iconColor;
	public WandType type;
	public WandSubType subType;
	public Rarity rarity;

	public int range;
	public Color magicColor;

	[ReadOnly]
	public float baseDamage, damage, maxDamage, castCost, castRate, chargeCost, chargeRate;

}
