﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using Photon.Pun;
using UnityEngine.SceneManagement;
#if UNITY_EDITOR
using UnityEditor;
#endif
using PhotonHashTable = ExitGames.Client.Photon.Hashtable;

public class PlayerBase : MonoBehaviourPunCallbacks {

	// References
	CharacterController characterController;
	BasicCameraBehaviour basicCameraBehaviour;
    Wand activeWand;
	Animator anim;
	GameObject drainObj;
	Vector3 spellScale = Vector3.one;
    RoundOverDisplay roundOverDisplay;


    // UI
    CanvasHandler canvasHandler;
	public RectTransform selectBorder, healthBar, manaBar;

	// Movement Vals
	public float speed = 8f, sprintSpeed = 15f, jumpSpeed = 10f, camSpeed = 8f, gravity = 20f;
	float heightOffset = 1f, yRot, startSpeed;
	int jumpOffset = 0; // this variable is here because there is a weird bug with the character controller that for one frame after they land, they are set to not grounded, before being perminantly grounded
	Vector3 playerRotation, pivotRotation;

	// Meter Vals
	public float playerHealth, wandMana, manaRegenVal, buildEnergy, refreshRate;
	public Color energyColor;

	// Pivot Points
	public Transform camPivot;
	public GameObject neckPivot;

    // Boolean Vals
	#if UNITY_EDITOR
    [ReadOnly]
	#endif
	public bool refreshed = true, inStorm = false, sprint = false, inAir = false, spawnDelay = true, defeated = false, jumped = false;

	// Velocities
	Vector3 forwardVelocity = Vector3.zero, horizontalVelocity = Vector3.zero, upwardVelocity = Vector3.zero, moveDirection = Vector3.zero;

	// Inventory
	public List<GameObject> inventory;
	public GameObject activeItem = null;
	public int activeItemNum = 0;

	// Magic Vals
	public GameObject spellPrefab;
	public GameObject orb, beam, scatter, burst, activeWandObj;
	public Transform spawnPoint;
	public float spellTravelSpeed = 20f, drainSpeed = 0.5f, gainedEnergy = 0f;
	[HideInInspector]
	public bool autoFire, charging;
	//[HideInInspector]
	public Transform magicSpawn, rightHand, leftHand;
	public LineRenderer energyBeam;
	//[HideInInspector]
	public List<Material> drainMats;
	public List<Energy> buildEnergies;

	// Private Magic Vals
	Ray magicRay;
	RaycastHit hit;
	Vector3 magicDir;
	float gScale;
	bool canDrain, casting;

    bool spectating; // this is used to tell that this player is the one being spectating

    // Initialize Values
    void Awake() {
        spectating = false;
        if (photonView && !photonView.IsMine) {
            inventory.Clear();
            return;
        }
        playerHealth = 100f;
        wandMana = 0f;
        buildEnergy = 0f;
        startSpeed = speed;
        canDrain = false;
        energyBeam.SetPosition(0, leftHand.transform.position);
        energyBeam.SetPosition(1, leftHand.transform.position);
        anim = GetComponent<Animator>();
        characterController = GetComponent<CharacterController>();
        inventory.Clear();
        drainMats.Clear();
        buildEnergies.Clear();
        buildEnergies = new List<Energy>();
        if (!spellPrefab)
            spellPrefab = orb;
        while (inventory.Count < 5)
            inventory.Add(null);

        if (!camPivot) {
            camPivot = GameObject.FindGameObjectWithTag("Camera Pivot").transform;
			ItemDisplays.itemDisplay.playerBase = this;
        }
		if (!spawnPoint && camPivot) {
			spawnPoint = camPivot;
		}
        if (!selectBorder) {
            selectBorder = GameObject.FindGameObjectWithTag("Selection Border").GetComponent<RectTransform>();
        }
        if (!healthBar) {
            healthBar = GameObject.FindGameObjectWithTag("Health Bar").GetComponent<RectTransform>();
        }
        if (!manaBar) {
            manaBar = GameObject.FindGameObjectWithTag("Mana Bar").GetComponent<RectTransform>();
        }
        if (!basicCameraBehaviour) {
            basicCameraBehaviour = camPivot.GetChild(0).GetComponent<BasicCameraBehaviour>();
        }
        if (!canvasHandler) {
            GameObject obj = GameObject.FindGameObjectWithTag("Game Manager");
            if (obj) {
                canvasHandler = obj.GetComponent<CanvasHandler>();
                if (!roundOverDisplay) {
                    roundOverDisplay = obj.GetComponent<RoundOverDisplay>();
                }
            }
        }

        if (PhotonNetwork.IsConnected) {
            PhotonHashTable update = new PhotonHashTable() { { "remaining", true } };
            PhotonNetwork.LocalPlayer.SetCustomProperties(update);
        }
    }

	void Update () {
        if (photonView && !photonView.IsMine) {
            if (spectating) {
                HandleCamPivot();
            }
            return;
        }
        CheckHealth ();
		CheckInput ();
		HandleCamPivot ();
		HandleMana ();
		AdjustBars ();

		if (activeItem != null) {
			anim.SetLayerWeight (1, 1f);
		} else {
			anim.SetLayerWeight (1, 0f);
		}
	}

    // trying to figure out how to get a rage quit / disconnect to drop their inventory
    //void OnDestroy() {
    //    if (SceneManager.GetActiveScene().name == "Launcher") {
    //        return;
    //    }
    //    if (photonView && photonView.Owner == null && !PhotonNetwork.IsMasterClient) {
    //        return;
    //    }

    //    if(!defeated) {
    //        DropAllItems();
    //    }
    //}

	void CheckHealth () {
		if (playerHealth > 100f)
			playerHealth = 100f;
		if (playerHealth <= 0f)
			Defeated ();
	}

	void CheckInput () {
		// Initial lock and unlock of curser, regardless of other inputs
		if (Input.GetMouseButtonDown (0) && !(roundOverDisplay && roundOverDisplay.IsRoundOver()))
			Screen.lockCursor = true;
		if (Input.GetKeyDown (KeyCode.Escape))
			Screen.lockCursor = false;

		// Check each input type
		MovementInput ();
		MagicInput ();
		InventoryInput ();

		// where to put you
		playerRotation.y = camPivot.eulerAngles.y;
		transform.eulerAngles = Vector3.Lerp (transform.eulerAngles, playerRotation, 1);
		Vector3 neckRot = new Vector3 (-transform.eulerAngles.y, 0f, 0f);
		neckPivot.transform.eulerAngles = transform.eulerAngles;

		// and you...
		energyBeam.SetPosition (0, leftHand.transform.position);
		if (!canDrain) {
			energyBeam.SetPosition (1, leftHand.transform.position);
		}

		magicRay = Camera.main.ViewportPointToRay (new Vector3 (0.5f, 0.5f, 0f));
	}

	void MovementInput () {
		// Set speed and movement animation vals
		sprint = (Input.GetKey (KeyCode.LeftShift)) ? true : false;
		speed = (sprint) ? sprintSpeed : startSpeed;
		float animFloat = (sprint) ? 2f : 1f;
		anim.SetFloat ("xDir", Input.GetAxis ("Horizontal") * animFloat);
		anim.SetFloat ("yDir", Input.GetAxis ("Vertical") * animFloat);

		// Jump handling and animation triggers
		if (characterController.isGrounded) {
			if (inAir) {
				inAir = false;
				anim.SetTrigger ("Land");
				moveDirection.y = 0f;
				jumped = false;
				jumpOffset = 0;
			}
			if (Input.GetButton ("Jump")) {
				inAir = true;
				anim.SetTrigger ("Jump");
				jumped = true;
				moveDirection.y = jumpSpeed;
			}
		} else {
			if (!jumped && jumpOffset > 1) {
				anim.SetTrigger ("Fall");
				inAir = true;
				jumped = true;
			}
			jumpOffset++;
			moveDirection.y -= gravity * Time.deltaTime;
		}

		// Set the movement velocities and move direction
		forwardVelocity = Input.GetAxis ("Vertical") * transform.forward * speed;
		horizontalVelocity = Input.GetAxis ("Horizontal") * transform.right * speed;
		upwardVelocity = transform.up * moveDirection.y;
		moveDirection = new Vector3(Input.GetAxis("Horizontal") * speed, moveDirection.y, Input.GetAxis("Vertical") * speed);

		// Animate and move the controller
		bool running = (Input.GetAxisRaw ("Horizontal") != 0f || Input.GetAxisRaw ("Vertical") != 0) ? true : false;
		characterController.Move((forwardVelocity + horizontalVelocity + upwardVelocity) * Time.deltaTime);
	}

	void MagicInput () {
		//Handle Resources
		EnergyDrain ();

		// Handle Magic Attacks
		if (activeWandObj) {
			activeWand = activeWandObj.GetComponent<WandScript> ().wand;
			SetActiveWandSpell (activeWand.type);
		
			if (Input.GetButtonDown ("Fire1") && wandMana > 0f && !canDrain) {
				if (activeWand.subType == WandSubType.Auto) {
					if (!autoFire)
						WandAnimsAndBools (true, false, true);
				} else if (activeWand.subType == WandSubType.Charged) {
					if (!charging)
						WandAnimsAndBools (false, true, true);
				} else {
					WandAnimsAndBools (false, false, true);
				}
			}
			if (Input.GetButtonUp ("Fire1"))
				WandAnimsAndBools (false, false, false);
		} else {
			activeWand = null;
		}
	}

	void SetActiveWandSpell (WandType type){
        spellPrefab = GetSpellPrefab(type);
	}

    GameObject GetSpellPrefab(WandType type) {
        switch (type) {
            case WandType.Beam:
                return beam;
            case WandType.Scatter:
                return scatter;
            case WandType.Orb:
                return orb;
            case WandType.Burst:
                return burst;
            default:
                return null;
        }
    }

	void WandAnimsAndBools (bool auto, bool charge, bool trigger) {
		autoFire = auto;
		charging = charge;
		anim.SetBool ("Auto", auto);
		anim.SetBool ("Charged", charge);
		if(trigger)
			anim.SetTrigger ("LeftClick");
	}

	public void AnimCast () {
        if (photonView && !photonView.IsMine)
            return;
		if (!canDrain){
			if (activeWand.subType == WandSubType.None)
				StartCoroutine (CastSpell (activeWand));
			else if (activeWand.subType == WandSubType.Auto)
				StartCoroutine (AutoCast (activeWand));
			else 
				StartCoroutine (ChargeCast (activeWand));
		}else{
			WandAnimsAndBools (false, false, false);
		}
	}

	IEnumerator AutoCast (Wand wand) {
		while (autoFire && wandMana > 0f) {
			StartCoroutine (CastSpell (wand));
			yield return new WaitForSeconds (0.25f);
		}
		yield return null;
	}

	IEnumerator ChargeCast (Wand wand) {
		float chargeTime = 0f;
		if (!charging) {
			StartCoroutine (CastSpell (wand));
		}
		while (charging) {
			yield return new WaitForSeconds (0.01f);
			wandMana = wandMana - 0.1f;
			chargeTime++;
			if (chargeTime == 10) {
				if (wand.damage < wand.maxDamage) {
					wand.damage = wand.damage + 1f;
					if (wand.type == WandType.Orb) {
						spellScale += new Vector3 (0.1f, 0.1f, 0.1f);
					}
					chargeTime = 0;
				}
			}
		}
		yield return null;
	}

	IEnumerator CastSpell (Wand wand) {
		magicDir = magicRay.direction;
		Quaternion rot = Quaternion.FromToRotation (spellPrefab.transform.forward, magicDir);
		Vector3 castPoint = spawnPoint.position;
        float[] xRotations = new float[wand.spellNum];
        float[] yRotations = new float[wand.spellNum];
        for (int i = 0; i < wand.spellNum; i++) {
			if (wand.type == WandType.Scatter) {
                xRotations[i] = Random.Range(-0.05f, 0.05f);
                yRotations[i] = Random.Range(-0.05f, 0.05f);
                rot.x += xRotations[i];
                rot.y += yRotations[i];
            }
			GameObject spell = Instantiate (spellPrefab, castPoint, rot);
			if (wand.type == WandType.Orb) {
				spell.transform.localScale = spellScale;
			}
			spell.GetComponent<Rigidbody> ().velocity = spell.transform.forward * spellTravelSpeed;//magicDir
            spell.GetComponent<Projectile>().SetDamage(wand.damage);
            wandMana = wandMana - 1f;
			StartCoroutine (DestroyProjectile (spell));
			if (wand.type == WandType.Burst) {
				yield return new WaitForSeconds (0.05f);
			}
            if(photonView && wand.type != WandType.Scatter) {
                photonView.RPC("FireProjectile", 
                    RpcTarget.Others, 
                    wand.type, 
                    spellPrefab.transform.forward, 
                    castPoint, 
                    magicDir,
                    spellScale,
                    spell.transform.forward * spellTravelSpeed,
                    wand.damage
                );
            }
		}
        if (photonView && wand.type == WandType.Scatter) {
            photonView.RPC("FireScatter",
                RpcTarget.Others,
                wand.type,
                spellPrefab.transform.forward,
                castPoint,
                magicDir,
                spellScale,
                spellTravelSpeed,
                wand.damage,
                wand.spellNum,
                (object)xRotations,
                (object)yRotations
            );
        }
        wand.damage = wand.baseDamage;
		spellScale = Vector3.one;
	}

    [PunRPC]
    public void FireProjectile(int spellType, Vector3 fwd, Vector3 pos, Vector3 dir, Vector3 scl, Vector3 vel, float dmg) {
        WandType wandType = (WandType)spellType;
        GameObject spell = Instantiate(GetSpellPrefab(wandType), pos, Quaternion.FromToRotation(fwd, dir));
        if (wandType == WandType.Orb) {
            spell.transform.localScale = scl;
        }
        spell.GetComponent<Rigidbody>().velocity = vel;
        spell.GetComponent<Projectile>().SetDamage(dmg);
        StartCoroutine(DestroyProjectile(spell));
    }

    [PunRPC]
    public void FireScatter(int spellType, Vector3 fwd, Vector3 pos, Vector3 dir, Vector3 scl, float speed, float dmg, int projectileCount, float[] xRotations, float[] yRotations) {
        WandType wandType = (WandType)spellType;
        GameObject spellPrefab = GetSpellPrefab(wandType);
        Quaternion rot = Quaternion.FromToRotation(fwd, dir);
        for (int i = 0; i < projectileCount; i++) {
            if (wandType == WandType.Scatter) {
                rot.x += xRotations[i];
                rot.y += yRotations[i];
            }
            GameObject spell = Instantiate(spellPrefab, pos, rot);
            spell.GetComponent<Rigidbody>().velocity = spell.transform.forward * speed;
            spell.GetComponent<Projectile>().SetDamage(dmg);
            StartCoroutine(DestroyProjectile(spell));
        }
    }

    void EnergyDrain () {
		if (Input.GetButtonDown ("Fire2") && Physics.Raycast (magicRay, out hit)) {
			magicDir = (hit.point - spawnPoint.position).normalized;
			if (hit.collider != null) {
				if (hit.collider.gameObject.GetComponent<MaterialDrainValue> () && hit.distance <= 40f) {
					if (hit.collider.gameObject.GetComponent<MaterialDrainValue> ().gScale < 1f) {
						drainObj = hit.collider.gameObject;
						canDrain = true;
						anim.SetLayerWeight (2, 1f);
						anim.SetTrigger ("RightClick");
						anim.SetBool ("Drain", true);
					}
				}
			}
		}

		if (Input.GetButton ("Fire2")) {
			if (canDrain) {
				if (Physics.Raycast (magicRay, out hit)) {
					magicDir = (hit.point - spawnPoint.position).normalized;
					if (hit.collider.gameObject == drainObj && hit.distance <= 40f) {
						energyBeam.SetPosition (1, hit.point);
					} else {
						StopDrain ();
						return;
					}
					if (!drainObj.GetComponent<MaterialDrainValue> ().DrainColor (this))
						StopDrain ();
				} else {
					StopDrain ();
				}
			}
		}
		if (Input.GetButtonUp ("Fire2")) {
			if(canDrain)
				anim.SetBool ("Drain", false);
		}
	}

	public void StopDrain () {
		if (drainObj)
			drainObj.GetComponent<MaterialDrainValue> ().GetEnergy (this);
		
		canDrain = false;
		drainMats.Clear ();
		drainSpeed = 0f;
		drainObj = null;
		StartCoroutine (DrainBeGone ());
	}


	IEnumerator DrainBeGone () {
		float t = 0;
		float layerWeight = 1;
		while (t < 1) {
			t += Time.deltaTime * 3f;
			layerWeight = Mathf.Lerp (1, 0, t);
			anim.SetLayerWeight (2, layerWeight);
			yield return null;
		}
		yield return null;
	}

	void InventoryInput () {
		if (Input.GetKeyDown (KeyCode.Q) || Input.GetKeyDown (KeyCode.E)) {
			if (Input.GetKeyDown (KeyCode.Q)) {
				selectBorder.anchoredPosition = (activeItemNum == 0) ? new Vector2 (-40, 40) : new Vector2 (selectBorder.anchoredPosition.x - 50, 40);
				activeItemNum = (activeItemNum == 0) ? 4 : activeItemNum - 1;
			} else if (Input.GetKeyDown (KeyCode.E)) {
				selectBorder.anchoredPosition = (activeItemNum == 4) ? new Vector2 (-240, 40) : new Vector2 (selectBorder.anchoredPosition.x + 50, 40);
				activeItemNum = (activeItemNum == 4) ? 0 : activeItemNum + 1;
			}
			NewActiveItem (inventory[activeItemNum]);
            if(photonView && inventory[activeItemNum] && inventory[activeItemNum].GetPhotonView()) {
                photonView.RPC("SwitchItem", RpcTarget.OthersBuffered, photonView.ViewID, inventory[activeItemNum].GetPhotonView().ViewID);
            }
            else if(photonView && inventory[activeItemNum] == null) {
                photonView.RPC("SwitchItem", RpcTarget.OthersBuffered, photonView.ViewID, -1);
            }
        }

		if (Input.GetKeyDown (KeyCode.F) && activeItem != null) {
            DropItem (activeItem);
        }
	}

    void NewActiveItem(GameObject item) {
        if (activeItem != null) {
            activeItem.GetComponent<ItemScript>().isActive = false;
            activeItem.SetActive(false);
        }
        activeItem = item;
        if (activeItem != null) {
            activeItem.SetActive(true);
            activeItem.GetComponent<ItemScript>().isActive = true;
        }
        else {
            activeWandObj = null;
        }
    }

    void HandleMana () {
		if (refreshed) {
			if (wandMana >= 100f) {
				wandMana = 100f;
			} else if (wandMana < 0) {
				StartCoroutine (RefreshMana ());
			} else if(!charging && !autoFire) {
				wandMana = wandMana + manaRegenVal;
			}
		}
	}

	void HandleCamPivot () {
        if(!camPivot) {
            camPivot = GameObject.FindGameObjectWithTag("Camera Pivot").transform;
            return;
        }

		camPivot.position = new Vector3 (transform.position.x, transform.position.y + heightOffset, transform.position.z);
		pivotRotation += new Vector3 (-Input.GetAxis ("Mouse Y") * camSpeed, Input.GetAxis ("Mouse X") * camSpeed, 0f);

		if (pivotRotation.x > 45f) {
			pivotRotation.x = 45f;
		} else if (pivotRotation.x < -45f) {
			pivotRotation.x = -45f;
		}

		camPivot.eulerAngles = pivotRotation;
	}

	void AdjustBars () {
        if(healthBar)
		    healthBar.sizeDelta = new Vector2 (2.5f * playerHealth, healthBar.sizeDelta.y);
		if(manaBar)
            manaBar.sizeDelta = new Vector2 (2.5f * wandMana, manaBar.sizeDelta.y);
	}

	IEnumerator RefreshMana () {
		wandMana = 0f;
		refreshed = false;
		yield return new WaitForSecondsRealtime (refreshRate);
		refreshed = true;
		yield return null;
	}

    void DropAllItems() {
        for(int i = 0; i < inventory.Count; i++) {
            if(inventory[i])
                DropItem(inventory[i], 500);
        }
    }

	public void DropItem (GameObject item, float forceMagnitude = 0, Vector3 direction = new Vector3()) {

        // check if wand was the active one
        if (activeWandObj == item)
			activeWandObj = null;

		// set bools
		item.GetComponent<ItemScript>().pickedUp = false;
		item.GetComponent<ItemScript> ().isActive = false;
		item.GetComponent<SphereCollider> ().enabled = true;

		// reset transform values
		item.transform.parent = null;
		item.transform.localScale = new Vector3 (2.25f, 2.25f, 2.25f);
		item.transform.localEulerAngles = new Vector3 (-130f, 0f, 0f);
        
        
		// disassociate the item from the player
		item.GetComponent<ItemScript> ().myOwner = null;
		item.SetActive (true);
        int inventoryIndex = inventory.IndexOf(item);
        if (inventoryIndex >= 0)
            inventory[inventory.IndexOf(item)] = null;
        activeItem = null;

        // anything that should happen to the local player but not remote players
        if (photonView && !photonView.IsMine) {
            if(forceMagnitude > 0) {
                item.GetComponent<Rigidbody>().AddForce(direction * forceMagnitude, ForceMode.Acceleration);
            }
            inventory.RemoveAll(GameObject => GameObject == null);
            return;
        }

        PhotonView itemPhotonView;
        if (itemPhotonView = item.GetComponent<PhotonView>()) {
            Vector3 dir = direction;
            if (forceMagnitude > 0) {
                if (dir == Vector3.zero) {
                    dir = Quaternion.Euler(0, Random.Range(0, 360), 0) * Vector3.forward;
                    dir.y = 1;
                }
                item.GetComponent<Rigidbody>().AddForce(dir * forceMagnitude, ForceMode.Acceleration);
            }
            if(photonView.Owner != null)
                itemPhotonView.RPC("DropItem", RpcTarget.OthersBuffered, photonView.Owner.ActorNumber, itemPhotonView.ViewID, forceMagnitude, dir);
        }

        ItemDisplays.itemDisplay.RemoveItem (activeItemNum);
	}

	void Defeated () {
		if(!defeated){

            if (PhotonNetwork.IsConnected) {
                PhotonHashTable update = new PhotonHashTable() { { "remaining", false } };
                PhotonNetwork.LocalPlayer.SetCustomProperties(update);
            }
            DropAllItems();
            anim.SetLayerWeight (1, 0f);
			anim.SetLayerWeight (2, 0f);
			anim.SetTrigger("Defeat");
			defeated = true;
            StartCoroutine(SwitchToSpectatorMode());
		}
	}

    IEnumerator SwitchToSpectatorMode() {
        yield return new WaitForSeconds(3f);

        if(roundOverDisplay && !roundOverDisplay.IsRoundOver()) {
            Transform t = GetNewCameraTargetTransform();

            if (!t) {
                if (PhotonNetwork.IsConnected)
                    PhotonNetwork.Destroy(photonView);
                else
                    Destroy(gameObject);
                yield break;
            }

            t.GetComponent<PlayerBase>().spectating = true;
            basicCameraBehaviour.ChangeCameraTarget(t);

            if (canvasHandler)
                canvasHandler.SetCanvasAsActive(CanvasHandler.CanvasType.Spectator);
            else
                Debug.LogError("No Canvas Handler Set");
        }
        
        if (PhotonNetwork.IsConnected)
            PhotonNetwork.Destroy(photonView);
        else
            Destroy(gameObject);
    }

    Transform GetNewCameraTargetTransform() {
        if (!PhotonNetwork.IsConnected) {
            return null;
        }

        GameObject[] players = GameObject.FindGameObjectsWithTag("Player");
        for(int i = 0; i < players.Length; i++) {
            PhotonView view = players[i].GetPhotonView();
            if (view && !view.IsMine)
                return players[i].transform;
        }

        return null;
    }

    IEnumerator DestroyProjectile (GameObject magic) {
		yield return new WaitForSeconds (2f);
		Destroy (magic);
	}

    [PunRPC]
    public void SwitchItem(int playerID, int itemID) {
        if (photonView.ViewID != playerID) {
            return;
        }
        if(itemID == -1) {
            NewActiveItem(null);
            return;
        }
        foreach(GameObject obj in inventory) {
            if (!obj)
                continue;
            if(obj.GetPhotonView() && obj.GetPhotonView().ViewID == itemID) {
                NewActiveItem(obj);
                return;
            }
        }
    }

    public void DoDamage(float damage) {
        // do damage animations/sounds here

        if (SceneManager.GetActiveScene().name == "Launcher") {
            return;
        }
        // if the round is over don't do any damage
        if (roundOverDisplay && roundOverDisplay.IsRoundOver()) {
            return;
        }
        if (playerHealth - damage > 0)
            playerHealth -= damage;
        else
            playerHealth = 0; 
    }

    public void SetSpectating(bool set) {
        spectating = set;
    }
}
